/*
 * Copyright (c) 2003 Koninklijke Philips Electronics
 *
 * Memory allocation functions.
 *
 * Contents of this file are not for general export
 */

#ifndef MWSYSTEM_H_INCLUDED 
#define MWSYSTEM_H_INCLUDED

/*
 * This file defines the memory allocation functions for Microwindows.
 * It is useful when porting to platforms where Microwindows needs to
 * use a non-standard memory allocator.
 *
 * This file defines macros or functions with the following prototypes:
 *
 * void * GdMalloc(size_t new_size);
 * void * GdCalloc(size_t num, size_t size);
 * void * GdRealloc(void * block, size_t old_size, size_t new_size);
 * void * GdFree(void * block);
 * char * GdStrDup(const char * src);
 *
 * The default implementation is to use the standard malloc()/free().
 *
 * Note the extra "old_size" parameter to GdRealloc().
 */

/* for definitions of calloc, malloc, realloc, and free and file ops */

#if defined SH3_MESOS || defined BFIN_MESOS || defined SH2A_MESOS || defined ARM_A5_MESOS || defined ARM_A9_SOC_MESOS || defined I686_MESOS
#include <stdlib.h>

// excerpt of <mem_pu.h>

#define MW_MEM_ALLOC_MODULE_NANO_X		0

extern void * mem_alloc (size_t size, unsigned int moduleId);
extern void * mem_calloc (size_t elems, size_t elemSize, unsigned int moduleId);
extern void mem_free (void * memPtr, size_t size, unsigned int moduleId);
extern void * mem_realloc (void * memPtr, size_t oldSize, size_t newSize, unsigned int moduleId);
extern char *mem_strdup (const char *s);

#define GdMalloc(size)                  mem_alloc(size, MW_MEM_ALLOC_MODULE_NANO_X)
#define GdCalloc(num,size)              mem_calloc(num, size, MW_MEM_ALLOC_MODULE_NANO_X)
#define GdRealloc(addr,oldsize,newsize) mem_realloc(addr, oldsize, newsize, MW_MEM_ALLOC_MODULE_NANO_X)
#define GdFree(addr)                    mem_free(addr, 0, MW_MEM_ALLOC_MODULE_NANO_X)
#define GdFree2(addr,size)              mem_free(addr, size, MW_MEM_ALLOC_MODULE_NANO_X)
#define GdStrDup(string)                mem_strdup(string)


// excerpt of <dvFS_pu.h>

typedef void * dvFS_voidFile_type;

extern dvFS_voidFile_type * dvFS_fopen (const char * filename, const char * mode);
extern int dvFS_fseek (dvFS_voidFile_type * stream, long offset, int origin);
extern int dvFS_fread (void * buffer, int elem_size, int elem_count, dvFS_voidFile_type * stream);
extern int dvFS_fclose (dvFS_voidFile_type * stream);

#define GdFile    dvFS_voidFile_type
#define GdFile_p  dvFS_voidFile_type *
#define GdFOpen(path, mode)           dvFS_fopen(path, mode)
#define GdFRead(file, buffer, size)   dvFS_fread(buffer, 1, size, file)
#define GdFSeek(file, offset, whence) dvFS_fseek(file, offset, whence)
#define GdFClose(file)                dvFS_fclose(file)

#else
#include <stdlib.h>

#define GdMalloc(size)                  malloc((size))
#define GdCalloc(num,size)              calloc((num),(size))
#define GdRealloc(addr,oldsize,newsize) realloc((addr),(newsize))
#define GdFree(addr)                    free((addr))
#define GdFree2(addr,size)              free((addr))
#define GdStrDup(string)                strdup((string))

/* The user hase the option including ZLIB and being able to    */
/* directly read compressed .pcf files, or to omit it and save  */
/* space.  The following defines make life much easier          */
#define GdFile	FILE
#if defined HAVE_FNTGZ_SUPPORT || defined HAVE_PCFGZ_SUPPORT
#include <zlib.h>
	#define GdFile_p gzFile
	#define GdFOpen(path, mode)           gzopen(path, mode)
	#define GdFRead(file, buffer, size)   gzread(file, buffer, size)
	#define GdFSeek(file, offset, whence) gzseek(file, offset, whence)
	#define GdFClose(file)                gzclose(file)
#else
	#define GdFile_p  FILE *
	#define GdFOpen(path, mode)           	fopen(path, mode)
	#define GdFRead(file, buffer, size)   	fread(buffer, 1, size, file)
	#define GdFSeek(file, offset, whence) 	fseek(file, offset, whence)
	#define GdFClose(file)                	fclose(file)
#endif
#endif

/*
Hint:  To change old source code to use the memory allocator defined
in this file, do (or rather, read, understand, then do):

find . \( -path './demos' -prune \) -o \( -name '*.[ch]' -a \! -name mwsystem.h -a -print \) | \
xargs perl -i -pWe \
's/malloc(\s*\()/GdMalloc$1/g; s/calloc(\s*\()/GdCalloc$1/g; s/strdup(\s*\()/GdStrDup$1/g; s/free(\s*\()/GdFree$1/g;'

(this script is also in src/include/memfix.sh)
You have to fix calls to realloc() by hand, since the parameters are different.
*/
#endif
